=== modified file 'NEWS'
--- NEWS	2006-12-21 19:32:25 +0000
+++ NEWS	2006-12-29 07:53:17 +0000
@@ -118,6 +118,9 @@
     * Mention the revisionspec topic for the revision option help.
       (Wouter van Heyst, #31663)
 
+    * Allow plugins import from zip archives.
+      (Alexander Belchenko, #68124)
+
 
 bzr 0.13  2006-12-05
     

=== modified file 'bzrlib/plugin.py'
--- bzrlib/plugin.py	2006-11-10 05:57:17 +0000
+++ bzrlib/plugin.py	2006-12-29 07:53:17 +0000
@@ -113,6 +113,7 @@
     dirs.insert(0, os.path.dirname(plugins.__file__))
 
     load_from_dirs(dirs)
+    load_from_zips(dirs)
 
 
 def load_from_dirs(dirs):
@@ -183,3 +184,81 @@
                 ## import pdb; pdb.set_trace()
                 warning('Unable to load plugin %r from %r' % (name, d))
                 log_exception_quietly()
+
+
+def load_from_zips(zips):
+    """Load bzr plugins from zip archives with zipimport.
+    It's similar to load_from_dirs but plugins searched inside archives.
+    """
+    import zipfile
+    import zipimport
+
+    valid_suffixes = ('.py', '.pyc', '.pyo')    # only python modules/packages
+                                                # is allowed
+    for zip_name in zips:
+        if '.zip' not in zip_name:
+            continue
+        try:
+            ziobj = zipimport.zipimporter(zip_name)
+        except zipimport.ZipImportError:
+            # not a valid zip
+            continue
+        mutter('Looking for plugins in %r', zip_name)
+
+        # use zipfile to get list of files/dirs inside zip
+        z = zipfile.ZipFile(ziobj.archive)
+        namelist = z.namelist()
+        z.close()
+
+        if ziobj.prefix:
+            prefix = ziobj.prefix.replace('\\','/')
+            ix = len(prefix)
+            namelist = [name[ix:]
+                        for name in namelist
+                        if name.startswith(prefix)]
+
+        mutter('Names in archive: %r', namelist)
+
+        for name in namelist:
+            if not name or name.endswith('/'):
+                continue
+
+            head, tail = os.path.split(name)
+            if '/' in head:
+                # we don't need looking in subdirectories
+                continue
+
+            base, suffix = os.path.splitext(tail)
+            if suffix not in valid_suffixes:
+                continue
+
+            if base == '__init__':
+                # package
+                plugin_name = head
+            elif head == '':
+                # module
+                plugin_name = base
+            else:
+                continue
+
+            if not plugin_name:
+                continue
+            if getattr(plugins, plugin_name, None):
+                mutter('Plugin name %s already loaded', plugin_name)
+                continue
+
+            try:
+                plugin = ziobj.load_module(plugin_name)
+                setattr(plugins, plugin_name, plugin)
+                mutter('Load plugin %s from zip %r', plugin_name, zip_name)
+            except zipimport.ZipImportError, e:
+                mutter('Unable to load plugin %r from %r: %s',
+                       plugin_name, zip_name, str(e))
+                continue
+            except KeyboardInterrupt:
+                raise
+            except Exception, e:
+                ## import pdb; pdb.set_trace()
+                warning('Unable to load plugin %r from %r'
+                        % (name, zip_name))
+                log_exception_quietly()

=== modified file 'bzrlib/tests/test_plugins.py'
--- bzrlib/tests/test_plugins.py	2006-12-21 15:15:19 +0000
+++ bzrlib/tests/test_plugins.py	2006-12-29 07:53:17 +0000
@@ -22,6 +22,7 @@
 
 import os
 from StringIO import StringIO
+import zipfile
 
 import bzrlib.plugin
 import bzrlib.plugins
@@ -196,3 +197,31 @@
             # remove the plugin 'myplug'
             if getattr(bzrlib.plugins, 'myplug', None):
                 delattr(bzrlib.plugins, 'myplug')
+
+
+class TestPluginFromZip(TestCaseInTempDir):
+
+    def make_zipped_plugin(self, zip_name, filename):
+        z = zipfile.ZipFile(zip_name, 'w')
+        z.writestr(filename, PLUGIN_TEXT)
+        z.close()
+
+    def check_plugin_load(self, zip_name, plugin_name):
+        self.assertFalse(plugin_name in dir(bzrlib.plugins),
+                         'Plugin already loaded')
+        try:
+            bzrlib.plugin.load_from_zips([zip_name])
+            self.assertTrue(plugin_name in dir(bzrlib.plugins),
+                            'Plugin is not loaded')
+        finally:
+            # unregister plugin
+            if getattr(bzrlib.plugins, plugin_name, None):
+                delattr(bzrlib.plugins, plugin_name)
+
+    def test_load_module(self):
+        self.make_zipped_plugin('./test.zip', 'ziplug.py')
+        self.check_plugin_load('./test.zip', 'ziplug')
+
+    def test_load_package(self):
+        self.make_zipped_plugin('./test.zip', 'ziplug/__init__.py')
+        self.check_plugin_load('./test.zip', 'ziplug')

